==> 0.0.2.tar.gz: lib/transform.dart <==

library fields.transform;

Set<Transform> _transforms;

// Register the set of default transforms.
_registerDefaults() {
  if(_transforms == null) {
    _transforms = new Set<Transform>();
    assert(register(const _IntToNumTransform()));
    assert(register(const _NumToIntTransform()));
    assert(register(const _StringToIntTransform()));
    assert(register(const _StringToDoubleTransform()));
  }
}

/// Register the given [transform].
/// Returns _true_ if there was no previously registered transform for the same 
/// source and target type, else returns _false_.  In all cases the [transform]
/// becomes the single registered transform for its source and target type.
bool register(Transform transform) {
  final had = canTransform(transform.sourceType, transform.targetType);
  if (had) {
    _transforms.removeWhere((t) => t.sourceType == transform.sourceType 
        && t.targetType == transform.targetType);
  }
  _transforms.add(transform);
  return !had;
}

/// Unregister the given [transform].
/// Returns _true_ if the [transform] was registered, else returns _false_.
bool unregister(Transform transform) => _transforms.remove(transform);

/// Check if a transform from the [sourceType] to the [targetType] can be 
/// performed.  Returns _true_ if such a transform is possible, else _false_.
bool canTransform(Type sourceType, Type targetType) {
  if (sourceType == targetType) return true;
  _registerDefaults();
  return _transforms.any(
      (t) => t.sourceType == sourceType && t.targetType == targetType);
}

/// Transform the given [value] from its [sourceType] to a [targetType].
/// Returns the transformed value.
transform(value, Type sourceType, Type targetType) {
  if (sourceType == targetType) return value;
  _registerDefaults();
  return _transforms.firstWhere(
      (t) => t.sourceType == sourceType && t.targetType == targetType)(value);
}

/// An error encountered during a transform.
class TransformError implements Error {
  /// The caught object that caused the transform to err; may be `null` if none.
  final cause;
  /// A description of this transform error.
  final String message;
  /// Constructs a new transform error with the optional [cause] and [message].
  TransformError({this.cause: null, this.message: ''});
  String toString() => 'TransformError: $message |> $cause';
}

/// A one-directional transform from type [S] to type [T].
abstract class Transform<S, T> {
  /// The source type of this transform.
  Type get sourceType;
  /// The target type of this transform.
  Type get targetType;
  const Transform();
  /// Call this transform on [value].  Returns the transformed value.
  T call(S value);
}

// -----------------------------------------------------------------------------
// Default Transforms.
// -----------------------------------------------------------------------------

class _IntToNumTransform extends Transform<int, num> {
  Type get sourceType => int;
  Type get targetType => num;
  const _IntToNumTransform();
  call(value) => value;
}

class _NumToIntTransform extends Transform<num, int> {
  Type get sourceType => num;
  Type get targetType => int;
  const _NumToIntTransform();
  call(value) => value.toInt();
}

class _StringToIntTransform extends Transform<String, int> {
  Type get sourceType => String;
  Type get targetType => int;
  const _StringToIntTransform();
  call(value) {
    try {
      return int.parse(value);
    } on FormatException catch(e) {
      throw new TransformError(cause: e);
    }
  }
}

class _StringToDoubleTransform extends Transform<String, double> {
  Type get sourceType => String;
  Type get targetType => double;
  const _StringToDoubleTransform();
  call(value) {
    try {
      return double.parse(value);      
    } on FormatException catch(e) {
      throw new TransformError(cause: e);
    }
  }
}

==> 0.0.4.tar.gz: lib/transform.dart <==
library fields.transform;

/// Check if a transform from the [sourceType] to the [targetType] can be 
/// performed.  Returns _true_ if such a transform is possible, else _false_.
bool canTransform(Type sourceType, Type targetType) {
  if (sourceType == targetType) return true;
  return Transform._transforms.any(
      (t) => t.sourceType == sourceType && t.targetType == targetType);
}

/// Transform the given [value] from its [sourceType] to a [targetType].
/// Returns the transformed value.
transform(value, Type sourceType, Type targetType) {
  if (sourceType == targetType) return value;
  return Transform._transforms.firstWhere(
      (t) => t.sourceType == sourceType && t.targetType == targetType)(value);
}

/// An error encountered during a transform.
class TransformError extends Error {
  /// The caught object that caused the transform to err; may be `null` if none.
  final cause;
  /// A description of this transform error.
  final String message;
  /// Constructs a new transform error with the optional [cause] and [message].
  TransformError({this.cause: null, this.message: ''});
  String toString() => 'TransformError: $message |> $cause';
}

/// A one-directional transform from type [S] to type [T].
abstract class Transform<S, T> {
  static final Set<Transform> _transforms = new Set<Transform>();
  
  static void register(Transform transform) => _transforms.add(transform);
  
  /// Unregister the given [transform].
  /// Returns _true_ if the [transform] was registered, else returns _false_.
  static bool unregister(Transform transform) => _transforms.remove(transform);
  
  static void unregisterAll() => _transforms.clear();
  
  /// The source type of this transform.
  final Type sourceType = S;
  /// The target type of this transform.
  final Type targetType = T;
  const Transform();
  /// Call this transform on [value].  Returns the transformed value.
  T call(S value);
}
